@@ -4,8 +4,8 @@
 ;;               2022–2025 anki-editor contributors
 
 ;; Author: Lei Tan
-;; Package-Version: 20251107.1231
-;; Package-Revision: 9289601129d4
+;; Package-Version: 20251222.1345
+;; Package-Revision: 59d75f6b6b63
 ;; URL: https://github.com/anki-editor/anki-editor
 ;; Package-Requires: ((emacs "29.1"))
 
@@ -245,17 +245,21 @@ The api is borrowed from request.el."
         (set-buffer-multibyte t)
         (insert data)))
     (unwind-protect
-        (with-current-buffer responsebuf
-          (apply #'call-process "curl" nil t nil (list
-                                                  url
-                                                  "--silent"
-                                                  "-X" type
-                                                  "--data-binary"
-                                                  (concat "@" tempfile)))
+        (unless (ignore-error json-end-of-file
+                  (with-current-buffer responsebuf
+                    (apply #'call-process "curl" nil t nil (list
+                                                        url
+                                                        "--silent"
+                                                        "-X" type
+                                                        "--data-binary"
+                                                        (concat "@" tempfile)))
 
-          (goto-char (point-min))
-          (when success
-            (apply success (list :data (funcall parser)))))
+                    (goto-char (point-min))
+                    (when success
+                      (apply success (list :data (funcall parser))))
+                    t))
+          (error (concat "Failed to connect to Anki.  "
+                         "Is Anki running with the AnkiConnect add-on enabled?")))
       (kill-buffer responsebuf)
       (delete-file tempfile))))
 
@@ -671,6 +675,7 @@ format the string, see `anki-editor--export-string'."
 (defconst anki-editor-prop-failure-reason "ANKI_FAILURE_REASON")
 (defconst anki-editor-prop-default-note-type "ANKI_DEFAULT_NOTE_TYPE")
 (defconst anki-editor-prop-swap-two-fields "ANKI_SWAP_TWO_FIELDS")
+(defconst anki-editor-prop-no-subheading-fields "ANKI_NO_SUBHEADING_FIELDS")
 (defconst anki-editor-org-tag-regexp "^\\([[:alnum:]_@#%]+\\)+$")
 
 (cl-defstruct anki-editor-note
@@ -1057,7 +1062,9 @@ and else from variable `anki-editor-prepend-heading'."
          (level (org-current-level))
          (content-before-subheading
           (anki-editor--note-contents-before-subheading))
-         (subheading-fields (anki-editor--build-fields))
+         (no-subheading-fields (org-entry-get nil anki-editor-prop-no-subheading-fields))
+         (subheading-fields (and (not no-subheading-fields)
+                                 (anki-editor--build-fields)))
          (field-swap
           (if (member note-type
                       (or (anki-editor--entry-get-multivalued-property-with-inheritance
@@ -1071,6 +1078,7 @@ and else from variable `anki-editor-prepend-heading'."
                                           level
                                           prepend-heading
                                           field-swap))
+         (fields (anki-editor--expand-attachment-links fields))
          ;; Sorting fields not necessary for Anki, but it removes
          ;; randomness which breaks our tests.
          (fields (sort fields (lambda (a b) (string< (car a) (car b))))))
@@ -1084,6 +1092,30 @@ and else from variable `anki-editor-prepend-heading'."
                            :hash hash
                            :marker (point-marker))))
 
+(defun anki-editor--expand-attachment-links (fields)
+  "Expand possible \"attachment:\" hyperlinks in the FIELDS alist.
+
+The exporter itself does not process links to attachments. They are normally
+processed with `org-attach-expand-links' called as a hook before the exporter
+sees the buffer content. It is made work on an org-mode buffer, not on a string,
+hence the need to shadow the `org-attach-dir' function."
+  (let* ((attach-dir (org-attach-dir))
+         (expand-links (lambda (str)
+                         (cl-letf (((symbol-function 'org-attach-dir)
+                                    (lambda () attach-dir)))
+                           ;; Reusing single tmp buffer for all fields
+                           (erase-buffer)
+                           (insert str)
+                           (goto-char (point-min))
+                           (org-attach-expand-links nil)
+                           (buffer-string)))))
+    (if (not attach-dir)
+        fields
+      (with-temp-buffer
+        (org-mode)
+        (cl-loop for (name . value) in fields
+                 collect (cons name (funcall expand-links value)))))))
+
 (defun anki-editor--get-tags ()
   "Return list of tags of org entry at point."
   (let ((tags (anki-editor--entry-get-multivalued-property-with-inheritance
@@ -1378,7 +1410,10 @@ determine the scope:
 See doc string of `org-map-entries' for what these different options mean.
 
 If one fails, the failure reason will be set in property drawer
-of that heading."
+of that heading.
+
+Unchanged notes are skipped unless `anki-editor-force-update' is
+non-nil, which see."
   (interactive (list (cond
                       ((region-active-p) 'region)
                       ((equal current-prefix-arg '(4)) 'tree)
@@ -1483,7 +1518,9 @@ beginning of the current heading."
 
 If point is not at a heading with an `ANKI_NOTE_TYPE' property,
 go up one heading at a time, until heading level 1, and push the
-subtree associated with the first heading that has one."
+subtree associated with the first heading that has one.
+
+Unlike `anki-editor-push-notes', always pushes unconditionally."
   (interactive)
   (save-excursion
     (anki-editor--goto-nearest-note-type)
@@ -1494,7 +1531,8 @@ subtree associated with the first heading that has one."
     (message "Successfully pushed note at point to Anki.")))
 
 (defun anki-editor-push-new-notes (&optional scope)
-  "Push note entries without ANKI_NOTE_ID in SCOPE to Anki."
+  "Push note entries without ANKI_NOTE_ID in SCOPE to Anki.
+Calls `anki-editor-push-notes', which see."
   (interactive)
   (anki-editor-push-notes scope (concat anki-editor-prop-note-id "=\"\"")))
 
